/**
 * 
 */
package sk.fiit.jim.agent.models;

import java.util.ArrayList;
import java.util.List;
import sk.fiit.jim.agent.AgentInfo;
import sk.fiit.robocup.library.geometry.Vector3D;


/**
 * Obsahuje informacie o hernzch situaciach - utocime, branime a podobne a taktiez metody
 * pre vytvorenie herej formacie
 * 
 *  TacticalInfo.java
 *  
 *@Title        Jim
 *@author       $Author: Filip $
 */
public class TacticalInfo {
	
	private boolean isOffensiveSituation = false;
	private boolean isDefSituation = false;
	private boolean isBallOurs = false;
	private boolean isBallTheir = false;
	private boolean isStartAttackSituation = false;
	private boolean isEnemyStartAttack = false;
	
	private Vector3D[] PositionInFormation;
	private Vector3D pos;
	private Vector3D relpos;
	
	private static TacticalInfo instance = new TacticalInfo();
	
	public static TacticalInfo getInstance(){
		return instance;
	}
	
	

	public boolean bIsBallNearestToMe(){
		List<Player> players = WorldModel.getInstance().getTeamPlayers();
		players.addAll(WorldModel.getInstance().getOpponentPlayers());
		double min = AgentInfo.HALF_FIELD_WIDTH*1000; //some relative big number

		//get minimal distance between ball and some other player
		for(Player p:players){
			if (p.getDistanceFromBall()<min){
				min = p.getDistanceFromBall();
			}
		}
		
		if (min < WorldModel.getInstance().getBall().getRelativePosition().getR()){
			return false; //some other player is nearer
		} else{
			return true;
		}
	}
	
	public boolean bIsBallNearestToMeInMyTeam(){
		List<Player> players = WorldModel.getInstance().getTeamPlayers();
		double min = AgentInfo.HALF_FIELD_WIDTH*1000; //some relative big number

		//get minimal distance between ball and some other player
		for(Player p:players){
			if (p.getDistanceFromBall()<min){
				min = p.getDistanceFromBall();
			}
		}
		
		if (min < WorldModel.getInstance().getBall().getRelativePosition().getR()){
			return false; //some other player is nearer
		} else{
			return true;
		}
	}
	
	public boolean bIsBallOurs(){
		List<Player> team = WorldModel.getInstance().getTeamPlayers();
		List<Player> oponent = WorldModel.getInstance().getOpponentPlayers();
		
		double minTeam = AgentInfo.HALF_FIELD_WIDTH*1000; //some relative big number
		double minOponent = AgentInfo.HALF_FIELD_WIDTH*1000; //some relative big number
		
		for(Player p:team){
			if (p.getDistanceFromBall()<minTeam){
				minTeam = p.getDistanceFromBall();
			}
		}
		
		if (minTeam>WorldModel.getInstance().getBall().getRelativePosition().getR()){
			minTeam = WorldModel.getInstance().getBall().getRelativePosition().getR();
		}
		
		for(Player p:oponent){
			if (p.getDistanceFromBall()<minOponent){
				minOponent = p.getDistanceFromBall();
			}
		}
		
		return minTeam<minOponent;
	}
	
	
	
	//--------------------------------------------------------------------------
	
	
	
	public void getPlaySituation(){
		if(!getIsOffensSituation())
			isStartAttackSituation();
		isOffensiveSituation();
		if(!getIsDefSituation());
			isEnemyStartAttack();
		isDefensiveSituation();
		
	}
	
	/*
	 * tato metoda zistuje, ci nas tim zaklada utok
	 * rozlisuje sa, ci zakladame utok pod tlakom alebo nie
	 * ak tim nema loptu v drzani zisti sa pomocou predikcie, ci ju do urciteho casu mat budu
	 */
	private void isStartAttackSituation(){
		
		List<Player> teamPlayers = WorldModel.getInstance().getTeamPlayers();
		String underPressure = "";
		
		for(Player teamPlayer: teamPlayers){
			isBallOurs(teamPlayer);
			if(isTMateOnOwnHalf(teamPlayer) && getIsBallOurs()){
				setIsStartAttackSituation(true);
				if(isUnderPressure(teamPlayer))
					underPressure = "Under pressure";
				else underPressure = "No pressure";
			}
			else {
				for(Player teamPlayer2: teamPlayers){
					if(Math.abs(teamPlayer2.getPrediction().getX() - WorldModel.getInstance().getBall().getPrediction().getX()) < 1 && 
					Math.abs(teamPlayer2.getPrediction().getY() - WorldModel.getInstance().getBall().getPrediction().getY()) < 1){
							setIsStartAttackSituation(true);
							if(isUnderPressure(teamPlayer2))
								underPressure = "Under pressure - predickia";
							else underPressure = "No pressure - predikcia";
						
					}else setIsStartAttackSituation(false);
				}
			}
		}
	
		if(getStartAttackSituation())
			System.out.print("Zakladame utok - " + underPressure + "\n");
	}
/*
 * podobna metoda ako pri isStartAttackSituation(), tentokrat pre superov tim
 * rovnaky princip ako u predchadzajucej
 */
private void isEnemyStartAttack(){
		
		ArrayList<Player> opponentPlayers = WorldModel.getInstance().getOpponentPlayers();
		String underPressure = "";
		
		for(Player opponentPlayer: opponentPlayers){
			isBallTheir(opponentPlayer);
			if(isEnemyOnOwnHalf(opponentPlayer) && getIsBallTheir()){
				setIsEnemyStartAttack(true);
				if(isUnderPressure2(opponentPlayer))
					underPressure = "Under pressure";
				else underPressure = "No pressure";
			}
			else {
				for(Player opponentPlayer2: opponentPlayers){
					if(Math.abs(opponentPlayer2.getPrediction().getX() - WorldModel.getInstance().getBall().getPrediction().getX()) < 1 && 
					Math.abs(opponentPlayer2.getPrediction().getY() - WorldModel.getInstance().getBall().getPrediction().getY()) < 1){
							setIsEnemyStartAttack(true);
							if(isUnderPressure2(opponentPlayer2))
								underPressure = "Under pressure - predickia";
							else underPressure = "No pressure - predikcia";
						
					}else setIsEnemyStartAttack(false);
				}
			}
		}
	
		if(getEnemyStartAttack())
			System.out.print("Super zaklada utok - " + underPressure + "\n");
	}
	
	private boolean isUnderPressure(Player teamPlayer){
		
		boolean isUnderPressure = false;
		
		ArrayList<Player> opponentPlayers = WorldModel.getInstance().getOpponentPlayers();
		
		for(Player opponentPlayer: opponentPlayers)
			if(Math.abs(opponentPlayer.getPosition().getX() - teamPlayer.getPosition().getX()) < 2 && 
			Math.abs(opponentPlayer.getPosition().getY() - teamPlayer.getPosition().getY()) < 2)
				isUnderPressure = true;
		
		return isUnderPressure;
	}
	
private boolean isUnderPressure2(Player opponentPlayer){
		
		boolean isUnderPressure = false;
		
		List<Player> teamPlayers = WorldModel.getInstance().getTeamPlayers();
		
		for(Player teamPlayer: teamPlayers)
			if(Math.abs(opponentPlayer.getPosition().getX() - teamPlayer.getPosition().getX()) < 2 && 
			Math.abs(opponentPlayer.getPosition().getY() - teamPlayer.getPosition().getY()) < 2)
				isUnderPressure = true;
		
		return isUnderPressure;
	}
	
	private boolean getStartAttackSituation(){
		return (this.isStartAttackSituation);
	}
	
	private void setIsEnemyStartAttack(boolean isEnemyStartAttack){
		if(this.isEnemyStartAttack != isEnemyStartAttack)
			this.isEnemyStartAttack = isEnemyStartAttack;
	}
	
	private boolean getEnemyStartAttack(){
		return (this.isEnemyStartAttack);
	}
	
	private void setIsStartAttackSituation(boolean isStartAttackSituation){
		if(this.isStartAttackSituation != isStartAttackSituation)
			this.isStartAttackSituation = isStartAttackSituation;
	}
	/*
	 * metoda zistuje, ci nas tim utoci a taktiez pomer nasich utocnikov k obrancom
	 */
	private void isOffensiveSituation(){
		
		List<Player> teamPlayers = WorldModel.getInstance().getTeamPlayers();
			
		if(getTMatesAttackers() != 0){
			if(getIsBallOurs()){
				if(this.getIsOffensSituation() == false)
					setIsOffensSituation(true);
			}else {
				for(Player teamPlayer: teamPlayers){
					if(Math.abs(teamPlayer.getPrediction().getX() - WorldModel.getInstance().getBall().getPrediction().getX()) < 1 && 
					Math.abs(teamPlayer.getPrediction().getY() - WorldModel.getInstance().getBall().getPrediction().getY()) < 1){
						if(this.getIsOffensSituation() == false)
							setIsOffensSituation(true);
					}else setIsOffensSituation(false);
				}
			}
		}else setIsOffensSituation(false);
		
		if(this.isOffensiveSituation)
			System.out.print("Utocime: " + getAttackType() + "\n");
		
	}
	
	/*
	 * metoda zistuje, ci nas tim brani a taktiez pomer nasich obrancov k superovym utocnikom
	 */
	private void isDefensiveSituation(){
		
		ArrayList<Player> opponentPlayers = WorldModel.getInstance().getOpponentPlayers();
			
		if(getEnemyAttackers() != 0){
			if(getIsBallTheir()){
				if(this.getIsDefSituation() == false)
					setIsDefSituation(true);
			}else setIsDefSituation(false);
		}else {
			for(Player opponentPlayer: opponentPlayers){
				if(Math.abs(opponentPlayer.getPrediction().getX() - WorldModel.getInstance().getBall().getPrediction().getX()) < 1 && 
				Math.abs(opponentPlayer.getPrediction().getY() - WorldModel.getInstance().getBall().getPrediction().getY()) < 1){
					if(this.getIsOffensSituation() == false)
						setIsOffensSituation(true);
				}else setIsOffensSituation(false);
			}
		}
		if(this.isDefSituation)
			System.out.print("Branime:" + getDefType() + "\n");
	}
	
	private String getAttackType(){
		
		return (getTMatesAttackers()+ " vs " + getEnemyDefenders() + "\n");
	}
	
	private String getDefType(){
		return (getTMatesDefenders() + " vs " + getEnemyAttackers() + "\n");
	}
	
	private boolean getIsOffensSituation(){
		return (this.isOffensiveSituation);
	}
	
	private boolean getIsDefSituation(){
		return (this.isDefSituation);
	}
	
	private void setIsOffensSituation(boolean isOffensiveSituation){
		if(this.isOffensiveSituation != isOffensiveSituation)
			this.isOffensiveSituation = isOffensiveSituation;
	}
	
	private void setIsDefSituation(boolean isDefSituation){
		if(this.isDefSituation != isDefSituation)
			this.isDefSituation = isDefSituation;
	}
	
	private int getEnemyDefenders(){
		
		ArrayList<Player> opponentPlayers = WorldModel.getInstance().getOpponentPlayers();
		
		int defenders = 0;
		
		for(Player opponentPlayer: opponentPlayers)
			if(isEnemyOnOwnHalf(opponentPlayer))
					defenders++;
		
		return defenders;
	}
	
	private boolean isEnemyOnOwnHalf(Player opponentPlayer){
		
		boolean onOwnHalf = false;
		
		if(opponentPlayer.getPosition().getX() < 12 && opponentPlayer.getPosition().getX() > 0)
			onOwnHalf = true;
				
		return onOwnHalf;
	}
	
	private int getTMatesDefenders(){
		
		ArrayList<Player> opponentPlayers = WorldModel.getInstance().getOpponentPlayers();
		List<Player> teamPlayers = WorldModel.getInstance().getTeamPlayers();
		int defenders = 0;
		
		if(AgentModel.getInstance().getPosition().getX() < 0 && AgentModel.getInstance().getPosition().getX() > -12)
			defenders++;
				
		for(Player teamPlayer: teamPlayers)
			for(Player opponentPlayer: opponentPlayers)
				if(isTMateOnOwnHalf(teamPlayer) && (teamPlayer.getPosition().getX() < opponentPlayer.getPosition().getX())){
					defenders++;
				}
		
		return defenders;
	}
	
	private boolean isTMateOnOwnHalf(Player teamPlayer){
		
		boolean onOwnHalf = false;
		
		if(teamPlayer.getPosition().getX() < 0 && teamPlayer.getPosition().getX() > -12)
			onOwnHalf = true;
				
		return onOwnHalf;
	}
	
	private int getTMatesAttackers(){
		
		List<Player> teamPlayers = WorldModel.getInstance().getTeamPlayers();
		int attackers = 0;
		
		if(AgentModel.getInstance().getPosition().getX() < 12 && AgentModel.getInstance().getPosition().getX() > 0)
			attackers++;
		
		if(Math.abs(AgentModel.getInstance().getPosition().getX() - WorldModel.getInstance().getBall().getPosition().getX()) < 1 && 
		Math.abs(AgentModel.getInstance().getPosition().getY() - WorldModel.getInstance().getBall().getPosition().getY()) < 1)
			setIsBallOurs(true);
		
		for(Player teamPlayer: teamPlayers){
			if(teamPlayer.getPosition().getX() < 12 && teamPlayer.getPosition().getX() > 0){
				isBallOurs(teamPlayer);	
				attackers++;
			}
		}
		
		return attackers;
	}
	
	private int getEnemyAttackers(){
		
		List<Player> opponentPlayers = WorldModel.getInstance().getOpponentPlayers();
		int attackers = 0;
		
		for(Player opponentPlayer: opponentPlayers){
			if(opponentPlayer.getPosition().getX() < 0 && opponentPlayer.getPosition().getX() > -12){
				isBallTheir(opponentPlayer);
				attackers++;
			}
		}
		
		return attackers;
	}
	
	private void isBallOurs(Player teamPlayer){
		
		if((Math.abs(teamPlayer.getPosition().getX() - WorldModel.getInstance().getBall().getPosition().getX()) < 1) && 
		(Math.abs(teamPlayer.getPosition().getX() - WorldModel.getInstance().getBall().getPosition().getX()) < 1)){
			if(this.isBallOurs == false)
				setIsBallOurs(true);
			}else setIsBallOurs(false);
	}
		
	
	private void setIsBallOurs(boolean isBallOurs){
		if(this.isBallOurs != isBallOurs)
			this.isBallOurs = isBallOurs;
		
	}
	
	private boolean getIsBallOurs(){
		return (this.isBallOurs);
	}
	
	private void isBallTheir(Player opponentPlayer){

			if((Math.abs(opponentPlayer.getPosition().getX() - WorldModel.getInstance().getBall().getPosition().getX()) < 1) && 
				(Math.abs(opponentPlayer.getPosition().getY() - WorldModel.getInstance().getBall().getPosition().getY()) < 1)){
					if(this.isBallTheir == false)
						setIsBallTheir(true);
					}else setIsBallTheir(false);
	}
	
	private void setIsBallTheir(boolean isBallTheir){
		if(this.isBallTheir != isBallTheir)
			this.isBallTheir = isBallTheir;	
	}
	
	private boolean getIsBallTheir(){
		return (this.isBallTheir);
	}
	
	/*-------------------------------------------------------------------------------------------
	 * Vytvorenie formacie
	 */
	
	public void setMyFormPosition(){
		
		PositionInFormation = new Vector3D[7];
		double ball_pos = WorldModel.getInstance().getBall().getPosition().getX();
		
		if(ball_pos < 5.5 && ball_pos > -4.5){		
			PositionInFormation[0]=Vector3D.cartesian(ball_pos - 5, 0, 0);
			PositionInFormation[1]=Vector3D.cartesian(ball_pos - 1, 0, 0);
			PositionInFormation[2]=Vector3D.cartesian(ball_pos + 4, 0, 0);
			PositionInFormation[3]=Vector3D.cartesian(ball_pos - 5, -3, 0);
			PositionInFormation[4]=Vector3D.cartesian(ball_pos - 5, 3, 0);
			PositionInFormation[5]=Vector3D.cartesian(ball_pos - 1, -3, 0);
			PositionInFormation[6]=Vector3D.cartesian(ball_pos - 1, +3, 0);
		}else{
			if(WorldModel.getInstance().getBall().getPosition().getX()<=0){
				PositionInFormation[0]=Vector3D.cartesian(-9, 0, 0);
				PositionInFormation[1]=Vector3D.cartesian(-5, 0, 0);
				PositionInFormation[2]=Vector3D.cartesian(0, 0, 0);
				PositionInFormation[3]=Vector3D.cartesian(-9, -4, 0);
				PositionInFormation[4]=Vector3D.cartesian(-9, 4, 0);
				PositionInFormation[5]=Vector3D.cartesian(-5, -4, 0);
				PositionInFormation[6]=Vector3D.cartesian(-5, +4, 0);
			}
			else
				PositionInFormation[0]=Vector3D.cartesian(0, 0, 0);
				PositionInFormation[1]=Vector3D.cartesian(4, 0, 0);
				PositionInFormation[2]=Vector3D.cartesian(8, 0, 0);
				PositionInFormation[3]=Vector3D.cartesian(0, -4, 0);
				PositionInFormation[4]=Vector3D.cartesian(0, 4, 0);
				PositionInFormation[5]=Vector3D.cartesian(4, -4, 0);
				PositionInFormation[6]=Vector3D.cartesian(4, +4, 0);
		}
		
		for(int i=0;i < 7;i++){
			if(AgentInfo.getInstance().playerId == i+1)
				pos = PositionInFormation[i];
		}
		
	}
	
	public Vector3D getFormPosition(){
		setMyFormPosition();
		
		relpos = AgentModel.getInstance().relativize(pos);
		
		
		return relpos;
	}
	
	public boolean isInPositionArea(){
		if(Math.abs(AgentModel.getInstance().getPosition().getXYDistanceFrom(pos))<1.5)
			return true;
		else 
			return false;
	}
	
		
	public boolean isOnPosition(){
		if(Math.abs(AgentModel.getInstance().getPosition().getXYDistanceFrom(pos))<0.5)
			return true;
		else 
			return false;
	}
}
