/**
 * Name:    TurnAround.java
 * Created: Feb 26, 2012
 * 
 * @author: ivan
 */
package sk.fiit.testframework.trainer.testsuite.testcases.tournament;

import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

import sk.fiit.robocup.library.geometry.Vector3;
import sk.fiit.testframework.communication.agent.AgentData;
import sk.fiit.testframework.communication.agent.AgentJim;
import sk.fiit.testframework.communication.agent.AgentJim.TeamSide;
import sk.fiit.testframework.communication.agent.AgentManager;
import sk.fiit.testframework.parsing.models.PlayMode;
import sk.fiit.testframework.trainer.testsuite.TestCase;
import sk.fiit.testframework.trainer.testsuite.TestCaseResult;
import sk.fiit.testframework.worldrepresentation.models.Player;
import sk.fiit.testframework.worldrepresentation.models.SimulationState;

/**
 * TODO: Replace with a brief purpose of class / interface.
 * 
 * @author ivan
 *
 */
public class KickDistanceTest extends TestCase {
	private static Logger logger = Logger.getLogger(KickDistanceTest.class.getName());

	private AgentData agentData;
	private AgentJim agent;
	private Vector3 initPosBall;
	private boolean started = false;
	private double startTime;
	
	public KickDistanceTest() {
		super();
	}

	@Override
	public boolean init() {
		super.init();
		logger.info("KickDistanceTest init");
		try {
			initPosBall = new Vector3(0,0,0.2);// new Vector3(0.0,0,0.0);//			
			
			AgentManager.getManager().setAgentWaitTime(30000);
			agent = AgentManager.getManager().getAgent(1, TeamSide.LEFT.toString(), false);
			if (agent != null) {
				agentData = agent.getAgentData();
			} else {
				agentData = new AgentData(1, TeamSide.LEFT, "ANDROIDS");
			}
			logger.info("Test initialized - waiting for PlayOn mode.");
			started = false;
			return true;
		} catch (Exception ex) {
			logger.log(Level.WARNING, "Unable to initialize test", ex);
			return false;
		}

	}

	@Override
	public boolean isStopCriterionMet(SimulationState ss) {
		if (!started) {
			if (ss.getGameStateInfo().getPlayMode() == PlayMode.PlayOn.ordinal()) {
				try {
					logger.fine("Test measurement started");
					started = true;
					server.setBallPosition(initPosBall.asPoint3D());
					server.setAgentPosition(agentData, new Vector3(-0.7,0,0.4).asPoint3D());
					if (agent != null) {
						agent.invokeMove("kick_right");
					}
					startTime = getElapsedTime();
				} catch (IOException e) {
					logger.log(Level.FINE, "Error running test", e);
				}
			}
			return false;
		} else {
			if (!ss.getScene().isBallMoving() && ss.getScene().getBallLocation().getXYDistanceFrom(initPosBall)> 0.5) {
				return true;
			}
			if (getElapsedTime() - startTime > 30) return true;
			return false;
		}
	}

	@Override
	public TestCaseResult evaluate(SimulationState ss) {
		Player p = ss.getScene().getPlayers().get(0);
		double distance = ss.getScene().getBallLocation().getXYDistanceFrom(initPosBall);
		if (p.isOnGround()) distance = distance / 2;
		return new TestCaseResult(distance);
	}

	@Override
	public void destroy() {
		super.destroy();
		try {
			Thread.sleep(500);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		logger.fine("Test case destroyed");
	}	

}
