/**
 * Name:    TurnAround.java
 * Created: Feb 26, 2012
 * 
 * @author: ivan
 */
package sk.fiit.testframework.trainer.testsuite.testcases.tournament;

import java.util.logging.Level;
import java.util.logging.Logger;

import sk.fiit.robocup.library.geometry.Vector3;
import sk.fiit.testframework.parsing.models.PlayMode;
import sk.fiit.testframework.trainer.testsuite.TestCase;
import sk.fiit.testframework.trainer.testsuite.TestCaseResult;
import sk.fiit.testframework.worldrepresentation.models.Player;
import sk.fiit.testframework.worldrepresentation.models.SimulationState;

public class StandUpTest extends TestCase {

	private static Logger logger = Logger.getLogger(StandUpTest.class.getName());

	private double startTime = -1;
	private boolean started = false;
	private Vector3 lastLocation1;

	public StandUpTest() {
		super();
	}

	@Override
	public boolean init() {
		super.init();
		try {
			started = false;
			logger.fine("Test initialized - waiting for PlayOn mode.");
			return true;
		} catch (Exception ex) {
			logger.log(Level.WARNING, "Unable to initialize test", ex);
			return false;
		}

	}

	@Override
	public boolean isStopCriterionMet(SimulationState ss) {
		Player p = ss.getScene().getPlayers().get(0);
		if (!started) {
			if (ss.getGameStateInfo().getPlayMode() == PlayMode.PlayOn.ordinal()) {
				logger.fine("Test measurement started");
				started = true;
				if (!monitor.getSimulationState().getScene().getPlayers().get(0).isOnGround()) return true; // should be on ground
				startTime = getElapsedTime();
			}
			return false;
		} else {
			
			if (p.getBody().getHead().getTransformation().getValues()[14]>=0.45) { // uz je postaveny
				return true;
			}
			
			if (getElapsedTime() - startTime > 20) // ak presiel maximalny dany cas
				return true;
			return false;
		}
	}

	@Override
	public TestCaseResult evaluate(SimulationState ss) {
		Player p = ss.getScene().getPlayers().get(0);
		if (p.getBody().getHead().getTransformation().getValues()[14]>=0.45) { // nehybe sa
			return new TestCaseResult(getElapsedTime() - startTime);
		} else {
			return new TestCaseResult(20);
		}

	}

	@Override
	public void destroy() {
		super.destroy();
		try {
			Thread.sleep(100);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		logger.fine("Test case destroyed");
	}	

}
