/**
 * Name:    TurnAround.java
 * Created: Feb 26, 2012
 * 
 * @author: ivan
 */
package sk.fiit.testframework.trainer.testsuite.testcases.tournament;

import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.junit.internal.runners.statements.ExpectException;

import sk.fiit.robocup.library.geometry.Vector3;
import sk.fiit.testframework.communication.agent.AgentData;
import sk.fiit.testframework.communication.agent.AgentJim;
import sk.fiit.testframework.communication.agent.AgentJim.TeamSide;
import sk.fiit.testframework.communication.agent.AgentManager;
import sk.fiit.testframework.parsing.models.PlayMode;
import sk.fiit.testframework.trainer.testsuite.TestCase;
import sk.fiit.testframework.trainer.testsuite.TestCaseResult;
import sk.fiit.testframework.worldrepresentation.models.Player;
import sk.fiit.testframework.worldrepresentation.models.SimulationState;

/**
 * TODO: Replace with a brief purpose of class / interface.
 * 
 * @author ivan
 *
 */
public class TurnAroundTest extends TestCase {
	private static Logger logger = Logger.getLogger(TurnAroundTest.class.getName());

	private AgentData agentData;
	private AgentJim agent;
	private boolean started = false;
	private double startTime;
	private double startingNormalizedRotation;
	private double exptectedRotation;
	rotation currentroration;
	enum rotation {none, left, right}
	
	public TurnAroundTest() {
		super();
	}

	@Override
	public boolean init() {
		super.init();
		try {        	
			agent = AgentManager.getManager().getAgent(1, TeamSide.LEFT.toString(), false);
			if (agent != null) {
				agentData = agent.getAgentData();
			} else {
				agentData = new AgentData(1, TeamSide.LEFT, "ANDROIDS");
			}
			logger.fine("Test initialized - waiting for PlayOn mode.");
			started = false;
			currentroration = rotation.none;
			return true;
		} catch (Exception ex) {
			logger.log(Level.WARNING, "Unable to initialize test", ex);
			return false;
		}

	}

	@Override
	public boolean isStopCriterionMet(SimulationState ss) {
		if (!started) {
			if (ss.getGameStateInfo().getPlayMode() == PlayMode.PlayOn.ordinal()) {
				try {
					logger.fine("Test measurement started");
					started = true;
//					server.setBallPosition(new Vector3(1, 0, 0.4).asPoint3D());
//					server.setAgentPosition(agentData, new Vector3(0,0,0.4).asPoint3D());
					startingNormalizedRotation = ss.getScene().getPlayers().get(0).getNormalizedRotation();
					exptectedRotation = startingNormalizedRotation + 3.14;
					if (exptectedRotation >= 3.14) exptectedRotation = startingNormalizedRotation - 3.14;
					logger.log(Level.FINE, "starting rotation (radian) " +  startingNormalizedRotation + ", end is expecting (radians)" + exptectedRotation);
					if (agent != null) {
						agent.invokeMove("turn_right_cont_20");
					}
				} catch (IOException e) {
					logger.log(Level.FINE, "Error running test", e);
				}
				startTime = getElapsedTime();
			}
			return false;
		} else {
			/*
			Tá rotácia je v Player.getNormalizedRotation() (ono to už aj predtým
			bolo správne, akurát to getRotation 3.tam dávalo rotáciu v opačnom smere
			než používal agent), vracia rotáciu v rovnakej forme ako sa nachádza v
			dátach agenta ( AgentModel.getRotationZ() ), teda v radiánoch kde 0 je
			tam, kde je agent otočený k ľavému okraju (strana, ktorá je pri
			pozeraní sa na modrú bránu po ľavej ruke). Je to použité napríklad v
			tom grafickom zobrazení v test frameworku.
			*/
			
			Player p = ss.getScene().getPlayers().get(0);
			double actualPlayerRotation = p.getNormalizedRotation();
			
			if (getElapsedTime() - startTime > 2 && currentroration == rotation.none) {
				if (actualPlayerRotation < startingNormalizedRotation) {
					logger.log(Level.FINE, "rotation right, expecting (radians) " + exptectedRotation);
					currentroration = rotation.right;
				} else {
					logger.log(Level.FINE, "rotation left, expecting (radians) " + exptectedRotation);
					currentroration = rotation.left;
				}
			}
			
			switch(currentroration) {
			case left: {
				if (actualPlayerRotation >= exptectedRotation) {
					return true;
				}
				break;
			}
			case right: {
				if (actualPlayerRotation <= exptectedRotation) {
					return true;
				}
				break;
			}
			}
			if (getElapsedTime() - startTime > 100)
				return true;
			return false;
		}
	}

	@Override
	public TestCaseResult evaluate(SimulationState ss) {
		Player p = ss.getScene().getPlayers().get(0);
		return new TestCaseResult(getElapsedTime() - startTime);
	}

	@Override
	public void destroy() {
		super.destroy();
		try {
			Thread.sleep(500);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		logger.fine("Test case destroyed");
	}	

}
